function downloadFile(url, filename, headers) {
    return new Promise((resolve, reject) => {
        chrome.runtime.sendMessage({
            download: {
                url,
                filename,
                headers, //header格式  [{name: 'User-Agent-New', value: 'pan.baidu.com'}]
            }
        }, (response) => {
            resolve(response); //返回id
        });
    });
}

function fileToBase64(file) {
    return new Promise((resolve, reject) => {
        let reader = new FileReader();
        reader.readAsDataURL(file);
        reader.onload = function () {
            resolve(reader.result);
        };
        reader.onerror = function (error) {
            reject(error);
        };
    });
}

function toast(title, type) {
    let toast = Swal.mixin({
        toast: true,
        position: 'top',
        showConfirmButton: false,
        timer: 2500,
        timerProgressBar: false,
        didOpen: (toast) => {
            toast.addEventListener('mouseenter', Swal.stopTimer);
            toast.addEventListener('mouseleave', Swal.resumeTimer);
        }
    });
    toast.fire({title: title, icon: type});
}



function addStyle() {
    let idName = 'youxiaohoubox-inject-style';
    if (document.getElementById(idName)) return;
    let style = document.createElement('style');
    style.id = idName;
    let css = `
        .inject-title {font-size: 24px; font-weight: 700; margin: 0 0 20px; padding: 0 0 16px; border-bottom: 1px solid #eee;color: #333;}
        .inject-dialog {display:flex;align-items: center;background: rgba(0, 0 ,0 , 0.6);margin-top: 15px;height:440px;}
        .inject-dialog .left-panel {flex: 1;padding: 0 10px;text-align: center;}
        .inject-dialog .left-panel img {max-width: 100%;max-height: 420px;}
        .inject-dialog .right-panel {flex: 0 0 240px;background: #fff;padding-left: 15px;position: relative;height: 100%;}
        .inject-dialog .right-panel .result {overflow-y: auto;box-sizing: border-box;}
        .inject-dialog .right-panel .result::-webkit-scrollbar { width: 6px; height: 6px }
        .inject-dialog .right-panel .result::-webkit-scrollbar-track { border-radius: 0; background: none }
        .inject-dialog .right-panel .result::-webkit-scrollbar-thumb { background-color: rgba(85, 85, 85, .4) }
        .inject-dialog .right-panel .result::-webkit-scrollbar-thumb, ::-webkit-scrollbar-thumb:hover { border-radius: 5px; -webkit-box-shadow: inset 0 0 6px rgba(0, 0, 0, .2) }
        .inject-dialog .right-panel .result::-webkit-scrollbar-thumb:hover { background-color: rgba(85, 85, 85, .3) }
        .inject-dialog .right-panel .result .item {display: flex;margin: 15px 0;font-size: 12px;color: #333;}
        .inject-dialog .right-panel .result .dot {display: flex;align-items: center;justify-content: center;width: 16px;height: 16px;border-radius: 50%;background: #e5e5e5;flex: 0 0 16px;margin-right: 5px;user-select: none}
        .inject-dialog .right-panel .result .text {text-align: left;flex: 1;word-break: break-all;cursor: pointer;}
        .inject-dialog .right-panel .result .text:hover {color:#ff1c1c;}
        .inject-dialog .right-panel .inject-copy-button { border-top: 1px solid #ddd; cursor: pointer; text-align: center; position: absolute; bottom: 0; left: 15px; right: 0; border: 0; border-radius: .25em; background: initial; background-color: #2778c4; color: #fff; font-size: 14px; padding: 9px 15px; box-shadow: none; font-weight: 500;}
        .inject-dialog .right-panel .copy-success {color: #ff1c1c;}    
        .inject-dialog .right-panel .inject-copy-button:hover {background-image: linear-gradient(rgba(0, 0, 0, 0.1), rgba(0, 0, 0, 0.1))}
}
    `;
    style.innerHTML = css;
    document.head.appendChild(style);
}

function addListener(event, className, callback) {
    if (className === '') {
        document.addEventListener(event, (e) => {
            callback && callback(e);
        });
    } else {
        document.addEventListener(event, (e) => {
            if (e.target.classList.contains(className)) {
                callback && callback(e);
            }
        });
    }
}

function setClipboard(text) {
    if (navigator.clipboard) {
        navigator.clipboard.writeText(text);
    } else {
        let textarea = document.createElement('textarea');
        document.body.appendChild(textarea);
        textarea.style.position = 'fixed';
        textarea.style.clip = 'rect(0 0 0 0)';
        textarea.style.top = '10px';
        textarea.value = text;
        textarea.select();
        document.execCommand('copy', true);
        document.body.removeChild(textarea);
    }
}

function generateHTML(option) {
    let html = '', copy = '';
    let len = option.data.result.length;
    let imgStyle = '';
    html += `<div><h3 class="inject-title">${option.title}（${len}）</h3>`;
    if (option.imgWidth) {
        imgStyle = `width: ${option.imgWidth};height: ${option.imgWidth};`;
    }
    html += `<div class="inject-dialog" style="height: ${option.height || 440}"><div class="left-panel" ><img src="${option.data.image}" style="${imgStyle}"></div><div class="right-panel"><div class="result" style="height: calc(100% - 45px)">`;

    for (let i = 0; i < len; i++) {
        let item = '';
        if (option.itemKey) {
            item = option.data.result[i][option.itemKey];
        } else {
            item = option.data.result[i];
        }
        copy += item + '\r\n';
        html += `<div class="item">
                <div class="dot">${i + 1}</div><div class="text listener-copy" title="点击复制" data-copy="${item}">${item}</div>
               </div>`;
    }
    if (option.buttonType === 'copy') {
        html += `</div><div class="inject-copy-button listener-copy" data-copy="${copy}">复制全部</div></div></div></div>`;
    } else if (option.buttonType === 'download') {
        html += `</div><div class="inject-copy-button listener-download">下载图片</div></div></div></div>`;
    } else {
        html += `</div></div></div></div>`;
    }

    addListener('click', 'listener-copy', (e) => {
        let originText = e.target.innerText;
        setClipboard(e.target.dataset.copy);
        e.target.innerText = '已复制到剪切板';
        e.target.classList.add('copy-success');
        setTimeout(() => {
            e.target.innerText = originText;
            e.target.classList.remove('copy-success');
        }, 3000);
    });

    addListener('click', 'listener-download', (e) => {
        downloadFile(option.data.image, '二维码.png', []);
    });

    Swal.fire({
        html,
        width: option.width || '1000px',
        showConfirmButton: false,
        footer: '由 <a style="margin:0 5px;" href="https://www.youxiaohou.com/box.html" target="_blank">油小猴工具箱</a> 提供支持',
    });
}

function showEncodeDialog(data) {
    if (data && data.image) {
        generateHTML({
            data,
            itemKey: '',
            title: '二维码生成成功',
            buttonType: 'download',
            imgWidth: '240px',
            width: '700px',
            height: '320px',
        });
    } else {
        toast('二维码生成失败','error');
    }
}

function showDecodeDialog(data) {
    if (data.result && data.result.length > 0) {
        generateHTML({
            data,
            itemKey: '',
            title: '二维码解码成功',
            buttonType: 'copy',
        });
    } else {
        toast('二维码解码失败','error');
    }
}

function showVerifyDialog(data) {
    if (data.result) {
        Swal.fire({
            toast: true,
            position: 'top',
            title: '验证码识别成功',
            html: `<div style="display: flex;justify-content: center;"><div style="padding: 7px 10px; text-align: center; background: #eaeaea; border-radius: 5px; font-weight: 700; margin-bottom: 5px; box-shadow: inset 0px 0px 5px 1px #dadada;">${data.result}</div></div>`,
            confirmButtonText: '复制',
            cancelButtonText: '关闭',
            showCancelButton: true,
        }).then((res) => {
            if (res.isConfirmed) {
                setClipboard(data.result);
                toast('已复制到剪切板','success');
            }
        });
    } else {
        toast('验证码识别失败','error');
    }
}

function showTextDialog(data) {
    if (data.result && data.result.length > 0) {
        generateHTML({
            data,
            itemKey: 'words',
            title: '文字识别成功',
            buttonType: 'copy',
        });
    } else {
        toast('文字识别失败','error');
    }
}

function showTranslateDialog(data) {
    if (data.result) {
        Swal.fire({
            toast: true,
            position: 'top',
            title: '文字翻译成功',
            html: `<div style="display: flex;justify-content: center;"><div style="padding: 7px 10px; background: #eaeaea; border-radius: 5px; margin-bottom: 5px; box-shadow: inset 0px 0px 5px 1px #dadada;font-size: 14px;white-space: pre-line">${data.result}</div></div>`,
            confirmButtonText: '复制',
            cancelButtonText: '关闭',
            showCancelButton: true,
        }).then((res) => {
            if (res.isConfirmed) {
                setClipboard(data.result);
            }
        });
    } else {
        toast('文字翻译失败','error');
    }
}

function showOCRDialog() {
    Swal.fire({
        title: 'OCR识别',
        html: `<div style="font-size: 12px;color: #999;margin-bottom: 8px;text-align: center;">提示：在任意网页按下 <span style="font-weight: 700;">F2</span> 键可快速打开本窗口。</div>
                <div style="font-size: 14px;line-height: 22px;padding: 10px 0 5px;text-align: left;">
                  <div><b>第一步：</b>请输入图片地址或使用微信，QQ等截图后 Ctrl+V</div>
                  <input type="textarea" class="swal2-input listener-image" placeholder="请输入图片地址">
                  <div><b>第二步：</b>当下方出现图片时，在图片上右键-> 选择【油小猴工具箱】-> 图片 -> OCR识别。</div>
                  <div style="margin-top: 12px;" id="img-preview">
                    <div style="height: 200px;width: 100%;background: #e7e7e7;color: #999;display: flex;align-items: center;justify-content: center;border-radius:3px">请在上方输入图片地址或 Ctrl+V 粘贴图片</div>
                  </div>
                </div>`,
        showCloseButton: false,
        showConfirmButton: false,
    });

    addListener('input', 'listener-image', (e) => {
        let image = e.target.value;
        if (image) {
            document.getElementById('img-preview').innerHTML = `<img src="${image}" style="max-width: 100%;"/>`;
        }
    });

    addListener('paste', '', (e) => {
        let items = e.clipboardData && e.clipboardData.items;
        let file = null;
        if (items && items.length) {
            // 检索剪切板items
            for (let i = 0; i < items.length; i++) {
                if (items[i].type.indexOf('image') !== -1) {
                    file = items[i].getAsFile();
                    break;
                }
            }
        }
        if (file) {
            fileToBase64(file).then(base64 => {
                document.getElementById('img-preview').innerHTML = `<img src="${base64}" style="max-width: 100%;"/>`;
            });
        }
    });
}

/**
 * 方法一：使用注入方法传值
 */
if (request && request.action) {
    console.log(request);
    addStyle();
    switch (request.action) {
        case "encodeQRCode":
            showEncodeDialog(request.data);
            break;
        case "decodeQRCode":
            showDecodeDialog(request.data);
            break;
        case "identifyVerifyCode":
            showVerifyDialog(request.data);
            break;
        case "identifyText":
            showTextDialog(request.data);
            break;
        case "translateText":
            showTranslateDialog(request.data);
            break;
        case "openOCRDialog":
            showOCRDialog();
            break;
    }
}

/**
 * 方法二：使用监听方法传值
 * 需要在 mainfest.json 中 content_scripts 节点中添加本脚本
 */
/*chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
        //console.log(request);
        switch (request.action) {
            case "encodeQRCode":
                showEncodeDialog(request.data);
                break;
            case "decodeQRCode":
                showDecodeDialog(request.data);
                break;
            case "verifyCode":
                showVerifyDialog(request.data);
                break;
        }
        return true;
    }
);*/
