(function () {
    'use strict';

    let util = {
        getValue(key) {
            return new Promise((resolve, reject) => {
                chrome.storage.sync.get([key], (result) => {
                    if (chrome.runtime.lastError) {
                        return reject(chrome.runtime.lastError);
                    }
                    resolve(result[key]);
                });
            });
        },

        setValue(option) {
            chrome.storage.sync.set(option);
        },

        addStyle(id, tag, css) {
            tag = tag || 'style';
            let doc = document, styleDom = doc.getElementById(id);
            if (styleDom) return;
            let style = doc.createElement(tag);
            style.rel = 'stylesheet';
            style.id = id;
            tag === 'style' ? style.innerHTML = css : style.href = css;
            document.head.appendChild(style);
        },
    };

    let main = {
        async showSetting() {
            let currentValue = await util.getValue('mactypeCurrentVal');
            Swal.fire({
                title: '请选择清晰度',
                icon: 'info',
                input: 'range',
                showCancelButton: true,
                confirmButtonText: '保存',
                cancelButtonText: '还原',
                showCloseButton: true,
                inputLabel: '拖动滑块观察变化，数字越大字越清晰',
                customClass: {
                    popup: 'mactype-popup',
                },
                footer: '<div style="text-align: center;font-size: 1em">点击查看 <a href="https://www.youxiaohou.com/tool/install-mactype.html" target="_blank">使用说明</a>，配合 <a href="https://www.youxiaohou.com/tool/install-mactype.html#增强显示" target="_blank">XHei字体</a> 更清晰。',
                inputAttributes: {
                    min: 0,
                    max: 1,
                    step: 0.05
                },
                inputValue: currentValue
            }).then((res) => {
                util.setValue({'mactypeInit': true});
                if (res.isConfirmed) {
                    util.setValue({'mactypeCurrentVal': res.value});
                    this.changeStyle();
                }
                if (res.isDismissed && res.dismiss === "cancel") {
                    util.setValue({'mactypeCurrentVal': 0});
                    this.changeStyle();
                }
            });

            document.getElementById('swal2-input').addEventListener('change', (e) => {
                util.setValue({'mactypeCurrentVal': e.target.value});
                this.changeStyle();
            });
        },

        async generateStyle() {
            let val = await util.getValue('mactypeCurrentVal');
            return `
                .mactype-popup { font-size: 14px!important }
                .swal2-range input { -webkit-appearance: auto!important; appearance: auto;!important }
                *:not(pre) { -webkit-text-stroke: ${val}px !important; text-stroke: ${val}px !important }
                ::selection { color: #fff;background: #338fff }
            `;
        },

        async changeStyle() {
            document.getElementById('mactype-style').innerHTML = await this.generateStyle();
        },

        async addPluginStyle() {
            let style = await this.generateStyle();

            if (document.head) {
                util.addStyle('mactype-style', 'style', style);
            }

            const headObserver = new MutationObserver(async () => {
                util.addStyle('mactype-style', 'style', style);
            });
            headObserver.observe(document.head, {childList: true, subtree: true});
        },

        isTopWindow() {
            return window.self === window.top;
        },

        init() {
            document.addEventListener('DOMContentLoaded', async () => {
                this.isTopWindow() && !await util.getValue('mactypeInit') && this.showSetting();
            }, false);
            this.addPluginStyle();
        }
    };
    util.getValue('mactypeEnable').then(res => {
        res && main.init();
    });
})();
